@@ -604,6 +604,7 @@ void yf::FrontendNet::process(mp::Package &package) const
             yaz_log(YLOG_LOG, "metaproxy received SIGTERM");
             break; /* stop right away */
         }
+#ifndef WIN32
         if (m_p->m_stop_signo == SIGUSR1)
         {    /* just stop listeners and cont till all sessions are done*/
             yaz_log(YLOG_LOG, "metaproxy received SIGUSR1");
@@ -616,6 +617,7 @@ void yf::FrontendNet::process(mp::Package &package) const
                 yaz_daemon_stop();
             }
         }
+#endif
         int no = m_p->mySocketManager.getNumberOfObservers();
         if (no <= 1)
             break;
@@ -725,7 +727,9 @@ void yf::FrontendNet::set_ports(std::vector<Port> &ports)
 
     // Create yf::FrontendNet::ZAssocServer for each port
     size_t i;
-    for (i = 0; i<m_p->m_ports.size(); i++)
+    for (i = 0; i < m_p->m_ports.size(); i++)
+        m_p->az[i] = 0;
+    for (i = 0; i < m_p->m_ports.size(); i++)
     {
         // create a PDU assoc object (one per yf::FrontendNet::ZAssocServer)
         yazpp_1::PDU_Assoc *as = new yazpp_1::PDU_Assoc(&m_p->mySocketManager);
