@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2013 Index Data
+   Copyright (C) Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -44,7 +44,7 @@ namespace metaproxy_1
 
         virtual RoutePos *createpos() const;
         void start();
-        void stop();
+        void stop(int signo);
     private:
         boost::scoped_ptr<Rep> m_p;
     };
