@@ -727,6 +727,15 @@ const char *mp::wrbuf::buf()
     return wrbuf_buf(m_wrbuf);
 }
 
+const char *mp::wrbuf::c_str()
+{
+    return wrbuf_cstr(m_wrbuf);
+}
+
+const char *mp::wrbuf::c_str_null()
+{
+    return wrbuf_cstr_null(m_wrbuf);
+}
 
 /*
  * Local variables:
